//=======================================================================
//                         DE0 spiMaster
//
// File:
//   fm_top.v
//
// Abstract:
//   OpenCores spimaster wrapper
//
//  Created:
//    27 December 2013
//======================================================================
//
// Copyright (c) 2013, Kenji Ishimaru
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  -Redistributions of source code must retain the above copyright notice,
//   this list of conditions and the following disclaimer.
//  -Redistributions in binary form must reproduce the above copyright notice,
//   this list of conditions and the following disclaimer in the documentation
//   and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// Revision History
// $Date: 2013-12-06 16:28:16 +0900 (Fri, 06 Dec 2013) $
// $Rev: 4 $

module fm_spimaster_wrapper (
  // AVALON Slave Interface
  input           clk_core,
  input           rst_x,
  input  [3:0]    i_av_adr,
  input  [3:0]    i_av_be,
  input           i_av_r,
  output [31:0]   o_av_rd,
  input           i_av_w,
  input  [31:0]   i_av_wd,
  output          o_av_wait,
  // SPI
  input           clk_spi,
  output          spiClkOut,
  input           spiDataIn,
  output          spiDataOut,
  output          spiCS_n
);
  // avalon bridge
  wire        w_req;
  wire        w_wr;
  wire [5:0]  w_adrs;
  wire        w_ack;
  wire        w_be;
  wire [7:0]  w_wd;
  wire        w_rstr;
  wire [7:0]  w_rd;

  assign w_rstr = (w_req & !w_wr & w_ack);
   
fm_avalon #(.P_AVALON_ADR_WIDTH(4),
            .P_INTERNAL_ADR_WIDTH(6),
            .P_INTERNAL_BE_WIDTH(1),
            .P_INTERNAL_DATA_WIDTH(8)
           ) u_avalon (
  .clk_core(clk_core),
  .rst_x(rst_x),
  // AVALON bus
  .i_av_adr(i_av_adr),
  .i_av_be(i_av_be),
  .i_av_r(i_av_r),
  .o_av_rd(o_av_rd),
  .i_av_w(i_av_w),
  .i_av_wd(i_av_wd),
  .o_av_wait(o_av_wait),
  // internal side
  .o_req(w_req),
  .o_wr(w_wr),
  .o_adrs(w_adrs),
  .i_ack(w_ack),
  .o_be(w_be),
  .o_wd(w_wd),
  .i_rstr(w_rstr),
  .i_rd(w_rd)
);
   
spiMaster u_spiMaster(
  .clk_i(clk_core),
  .rst_i(~rst_x),
  .address_i({2'b0,w_adrs}),
  .data_i(w_wd),
  .data_o(w_rd),
  .strobe_i(w_req),
  .we_i(w_wr),
  .ack_o(w_ack),
  // SPI
  .spiSysClk(clk_spi),
  .spiClkOut(spiClkOut),
  .spiDataIn(spiDataIn),
  .spiDataOut(spiDataOut),
  .spiCS_n(spiCS_n)
);

endmodule
